/* AI Switch bank routing
 * Michael Robinson
 */
`timescale 1ns/10ps

/* Address switch bank routing module 
 * Input: [8:0] ai = address switches, [7:0] get routed
 *        mee_ = When halted, switches refer to memory/devices
 *        i1e_ = When halted, switches refer to first instruction word
 *        i2e_ = When halted, switches refer to second instruction word
 *        i3e_ = When halted, switches refer to third instruction word
 * Output: 
 *        ensr_ = when asserted, state register is on aiprg bus, when
 *                deasserted, this module writes to aiprg
 * Input/Output:
 *        [7:0] ablo = data address bus
 *        [7:0] aiprg = program address bus
 */
module airoute( ai, mee_, i1e_, i2e_, i3e_, 
                ensr_, ablo, aiprg );
input [8:0] ai;
input mee_, i1e_, i2e_, i3e_;
output ensr_;
output [7:0] ablo, aiprg;

wire ensr, ensr_;
wire [7:0] ai_truncated;

assign ensr = i1e_ & i2e_ & i3e_;
assign ensr_ = ~ensr;
assign ai_truncated = ai[7:0];

u74ls244 aiprgb( .d(ai_truncated), .q(aiprg), .oe_(ensr) );
u74ls244 aimem( .d(ai_truncated), .q(ablo), .oe_(mee_) );

endmodule