/* ALU unit 
 * Michael Robinson 
 */

/* ALU unit constructed from 4 bit ALUs
 * Input:
 *       [7:0] a, b = operands
 *       [3:0] op  = opcode
 * Output:
 *       [7:0] f = result of operation
 *       bit  = flag bit from result
 */
module aluunit( a, b, op, f, bit );
input [7:0] a, b;
input [3:0] op;
output [7:0] f;
output bit;

reg [7:0] f;
reg bit;

always @(a or b or op ) begin
  case( op )
    4'h0: f = a;
    4'h1: f = b;
    4'h2: f = a ^ b;
    4'h3: f = a + b;
    4'h4: f = 0;
    4'h5: f = a + 1;
    4'h6: f = a & b;
    4'h7: f = a | b;
    4'h8: f = ~a;
    4'h9: f = 0;
    4'ha: f = 0;
    4'hb: f = 0;
    4'hc: f = 0;
    4'hd: f = 0;
    4'he: f = 0;
    4'hf: f = 0;
  endcase

  if( f == 4'h0 ) bit = 1;
  else bit = 0;
end
endmodule