/* Timing sequencer 
 * Michael Robinson 
 */
`timescale 1ns/10ps

/* Timing sequencer module
 * Input:
 *        clk = system clock
 *        reset_ = systemwide reset
 *        run = run switch
 *        step = step switch
 *        pause_ = bus pause line (pauses counting)
 * Output:
 *        run_ = simply invert the run switch
 *        t1 = active during timestate 1
 *        t23 = active during timestate 2 and 3
 *        t3_ = active during timestate 3
 *        t4 = active during timestate 4
 */
module sequencer( clk, reset_, run, step, pause_,
                  run_, t1, t23, t3_, t4 );
input clk, reset_, run, step, pause_;
output run_, t1, t23, t3_, t4;

wire run_;
wire t1, t2, t23, t3_, t4, t3;
wire st23_, rt23_, rt23a_;
wire trueclk;

assign run_ = ~run;
assign trueclk = ( run & clk & pause_ ) | step;
assign rt23_ = rt23a_ & reset_;

u74ls74 state1( .d(t4), .q(t1), .s_(1), .r_(reset_), .clk(trueclk) );
u74ls74 state2( .d(t1), .q(t2), .q_(st23_), .s_(1), .r_(reset_), 
                .clk(trueclk) );
u74ls74 state3( .d(t2), .q(t3), .q_(t3_), .s_(1), .r_(reset_), .clk(trueclk) );
u74ls74 state4( .d(t3), .q(t4), .q_(rt23a_), .s_(reset_), .r_(1), 
                .clk(trueclk) );
u74ls74 state23( .d(0), .q(t23), .s_(st23_), .r_(rt23_), .clk(0));

endmodule
