/* Computer testbench */

`timescale 1ns/10ps
module top();

reg clk;
reg reset_, run, step;
reg write_, write_en_;
reg sre_, pre_, mee_, i1e_, i2e_, i3e_;
reg [7:0] di;
reg [8:0] ai;

wire [7:0] ddbus;
wire pause_, devwr_, devrd_;
wire [7:0] ablo, abhi;

initial begin
   $dumpfile("dumpfile.vcd");
   $dumpvars(0, top );

   reset_ = 0;
   clk = 0;
   run = 0; 
   step = 0;
   write_ = 1;
   write_en_ = 1;
   sre_ = 1;
   pre_ = 1;
   mee_ = 1;
   i1e_ = 1;
   i2e_ = 1;   
   i3e_ = 1;
   ai = 9'b000000000;
   di = 8'b00000000;

   #20 reset_ = 1;

   #20
   sre_ = 0;
   di = 8'h0;
   #10
   write_ = 0;
   #10
   write_ = 1;

   #20
   sre_ = 1;
   pre_ = 0;
   di = 8'h00;
   #10
   write_ = 0;
   #10
   write_ = 1;
   pre_ = 1;

   #20
   i1e_ = 0;
   ai = 9'b000000000;
   di = 8'b00001100; //Load IR with memory
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000001;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000001;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000000;
   di = 8'b00000001;
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   i1e_ = 0;
   ai = 9'b000000001;
   di = 8'b00001000; //Load acc with memory
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000000;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000010;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000001;
   di = 8'b00000010;
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   i1e_ = 0;
   ai = 9'b000000010;
   di = 8'b01000000;  // Invert acc 
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000001;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000011;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000010;
   di = 8'b00000011;
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   i1e_ = 0;
   ai = 9'b000000011;
   di = 8'b00101000; // Increment acc
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000000;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000100;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000011;
   di = 8'b00000100;
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   i1e_ = 0;
   ai = 9'b000000100;
   di = 8'b00011011; // Add acc to mem
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000001;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000001;  // If result != 0, try again
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000100;
   di = 8'b00000101; // If result == 0
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   i1e_ = 0;
   ai = 9'b000000101;
   di = 8'b00000110;  // Write result to PR
   #10 write_ = 0;
   #10 write_ = 1;
   i1e_ = 1;
   i2e_ = 0;
   di = 8'b00000000;
   #10 write_ = 0;
   #10 write_ = 1;
   i2e_ = 1;
   i3e_ = 0;
   di = 8'b00000101;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b100000101;
   di = 8'b00000101;
   #10 write_ = 0;
   #10 write_ = 1;
   i3e_ = 1;

   #20 mee_ = 0;
   ai = 9'b000000000;
   di = 8'h1;
   write_en_ = 0;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b000000001;
   di = 8'h5;
   #10 write_ = 0;
   #10 write_ = 1;
   ai = 9'b000000101;
   di = 8'h2;
   #10 write_ = 0;
   #10 write_ = 1;
   mee_ = 1;
   write_en_ = 1;

   #20 run = 1;

   #10000 $finish;
end

always #10 clk = ~clk;

comp4 testcomp( di, ai, run, reset_, clk, write_, step, write_en_, sre_, 
             pre_, mee_, i1e_, i2e_, i3e_,
             pause_, devwr_, devrd_, ablo, abhi, ddbus
             );

memmod datamem( ddbus, ablo, abhi, devwr_, devrd_, pause_ );

endmodule